@@ -70,28 +70,28 @@ class PermissionRoleManagerTestCase(TestCase):
         role6._users.add(user8)
 
         self.assertItemsEqual(Role.objects.filter_by_user(user1), [
-                role1, role2, role3, role4, role5, role6
+                role1,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user2), [
-                role1, role2, role3, role4, role5, role6
+                role1,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user3), [
-                role2,
+                role1, role2,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user4), [
-                role3, role4, role5,
+                role1, role3,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user5), [
-                role3, role4, role5,
+                role1, role3,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user6), [
-                role4,
+                role1, role3, role4,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user7), [
-                role5,
+                role1, role3, role5,
             ])
         self.assertItemsEqual(Role.objects.filter_by_user(user8), [
-                role6,
+                role1, role6,
             ])
 
     def test_get_all_permissions_of_user(self):
@@ -137,30 +137,28 @@ class PermissionRoleManagerTestCase(TestCase):
         role6._permissions.add(perm8)
 
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user1), [
-                perm1, perm2, perm3, perm4,
-                perm5, perm6, perm7, perm8,
+                perm1, perm2,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user2), [
-                perm1, perm2, perm3, perm4,
-                perm5, perm6, perm7, perm8,
+                perm1, perm2,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user3), [
-                perm3,
+                perm1, perm2, perm3,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user4), [
-                perm4, perm5, perm6, perm7,
+                perm1, perm2, perm4, perm5,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user5), [
-                perm4, perm5, perm6, perm7,
+                perm1, perm2, perm4, perm5,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user6), [
-                perm6,
+                perm1, perm2, perm4, perm5, perm6,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user7), [
-                perm7,
+                perm1, perm2, perm4, perm5, perm7,
             ])
         self.assertItemsEqual(Role.objects.get_all_permissions_of_user(user8), [
-                perm8,
+                perm1, perm2, perm8,
             ])
 
 
@@ -252,23 +250,22 @@ class PermissionRoleModelTestCase(TestCase):
         role6._permissions.add(perm8)
 
         self.assertItemsEqual(role1.permissions, [
-                perm1, perm2, perm3, perm4,
-                perm5, perm6, perm7, perm8
+                perm1, perm2
             ])
         self.assertItemsEqual(role2.permissions, [
-                perm3,
+                perm1, perm2, perm3,
             ])
         self.assertItemsEqual(role3.permissions, [
-                perm4, perm5, perm6, perm7
+                perm1, perm2, perm4, perm5,
             ])
         self.assertItemsEqual(role4.permissions, [
-                perm6,
+                perm1, perm2, perm4, perm5, perm6,
             ])
         self.assertItemsEqual(role5.permissions, [
-                perm7,
+                perm1, perm2, perm4, perm5, perm7,
             ])
         self.assertItemsEqual(role6.permissions, [
-                perm8,
+                perm1, perm2, perm8,
             ])
 
     def test_is_belong(self):
@@ -347,24 +344,31 @@ class PermissionRoleModelTestCase(TestCase):
         role4 = create_role('permission_test_role4', role3)
         role5 = create_role('permission_test_role5', role3)
         role6 = create_role('permission_test_role6', role1)
-        role1._users.add(user1, user2)
-        role2._users.add(user3)
-        role3._users.add(user4, user5)
-        role4._users.add(user6)
-        role5._users.add(user7)
-        role6._users.add(user8)
+        role1.users.add(user1, user2)
+        role2.users.add(user3)
+        role3.users.add(user4, user5)
+        role4.users.add(user6)
+        role5.users.add(user7)
+        role6.users.add(user8)
 
-        # actually role1 doesn't have user3
-        self.assertFalse(role1._users.filter(pk=user3.pk).exists())
-        # but user3 is in the subroles of role1 thus add_users doesn't add
-        role1.add_users(user3)
-        self.assertFalse(role1._users.filter(pk=user3.pk).exists())
-
-        # role2 doesn't have user2
-        self.assertFalse(role2._users.filter(pk=user2.pk).exists())
-        # and no subroles have user2 thus add_users add user2 to role2
-        role2.add_users(user2)
-        self.assertTrue(role2._users.filter(pk=user2.pk).exists())
+        self.assertItemsEqual(role1._users.all(), [
+                user1, user2,
+            ])
+        self.assertItemsEqual(role2._users.all(), [
+                user3,
+            ])
+        self.assertItemsEqual(role3._users.all(), [
+                user4, user5,
+            ])
+        self.assertItemsEqual(role4._users.all(), [
+                user6,
+            ])
+        self.assertItemsEqual(role5._users.all(), [
+                user7,
+            ])
+        self.assertItemsEqual(role6._users.all(), [
+                user8,
+            ])
 
     def test_remove_users(self):
         # role1           -- user1, user2
@@ -393,21 +397,61 @@ class PermissionRoleModelTestCase(TestCase):
         role4._users.add(user6)
         role5._users.add(user7)
         role6._users.add(user8)
+        role1.users.remove(user1, user2)
+        role2.users.remove(user3)
+        role3.users.remove(user4, user5)
+        role4.users.remove(user6)
+        role5.users.remove(user7)
+        role6.users.remove(user8)
 
-        # actually role1 doesn't have user3
-        self.assertFalse(role1._users.filter(pk=user3.pk).exists())
-        # thus remove_user doesn't remove even user3 is in subroles
-        role1.remove_users(user3)
-        self.assertFalse(role1._users.filter(pk=user3.pk).exists())
-        self.assertTrue(role1.users.filter(pk=user3.pk).exists())
+        self.assertItemsEqual(role1._users.all(), [])
+        self.assertItemsEqual(role2._users.all(), [])
+        self.assertItemsEqual(role3._users.all(), [])
+        self.assertItemsEqual(role4._users.all(), [])
+        self.assertItemsEqual(role5._users.all(), [])
+        self.assertItemsEqual(role6._users.all(), [])
+
+    def test_clear_users(self):
+        # role1           -- user1, user2
+        #   +- role2      -- user3
+        #   +- role3      -- user4, user5
+        #   |    +- role4 -- user6
+        #   |    +- role5 -- user7
+        #   +- role6      -- user8
+        user1 = create_user('permission_test_user1')
+        user2 = create_user('permission_test_user2')
+        user3 = create_user('permission_test_user3')
+        user4 = create_user('permission_test_user4')
+        user5 = create_user('permission_test_user5')
+        user6 = create_user('permission_test_user6')
+        user7 = create_user('permission_test_user7')
+        user8 = create_user('permission_test_user8')
+        role1 = create_role('permission_test_role1')
+        role2 = create_role('permission_test_role2', role1)
+        role3 = create_role('permission_test_role3', role1)
+        role4 = create_role('permission_test_role4', role3)
+        role5 = create_role('permission_test_role5', role3)
+        role6 = create_role('permission_test_role6', role1)
+        role1._users.add(user1, user2)
+        role2._users.add(user3)
+        role3._users.add(user4, user5)
+        role4._users.add(user6)
+        role5._users.add(user7)
+        role6._users.add(user8)
+        role1.users.clear()
+        role2.users.clear()
+        role3.users.clear()
+        role4.users.clear()
+        role5.users.clear()
+        role6.users.clear()
+
+        self.assertItemsEqual(role1._users.all(), [])
+        self.assertItemsEqual(role2._users.all(), [])
+        self.assertItemsEqual(role3._users.all(), [])
+        self.assertItemsEqual(role4._users.all(), [])
+        self.assertItemsEqual(role5._users.all(), [])
+        self.assertItemsEqual(role6._users.all(), [])
 
-        # role2 have user3
-        self.assertTrue(role2._users.filter(pk=user3.pk).exists())
-        # thus remove_user remove user3
-        role2.remove_users(user3)
-        self.assertFalse(role2._users.filter(pk=user3.pk).exists())
-        self.assertFalse(role2.users.filter(pk=user3.pk).exists())
-        self.assertFalse(role1.users.filter(pk=user3.pk).exists())
 
     def test_add_permissions(self):
         # role1           -- perm1, perm2
@@ -430,24 +474,32 @@ class PermissionRoleModelTestCase(TestCase):
         role4 = create_role('permission_test_role4', role3)
         role5 = create_role('permission_test_role5', role3)
         role6 = create_role('permission_test_role6', role1)
-        role1._permissions.add(perm1, perm2)
-        role2._permissions.add(perm3)
-        role3._permissions.add(perm4, perm5)
-        role4._permissions.add(perm6)
-        role5._permissions.add(perm7)
-        role6._permissions.add(perm8)
+        role1.permissions.add(perm1, perm2)
+        role2.permissions.add(perm3)
+        role3.permissions.add(perm4, perm5)
+        role4.permissions.add(perm6)
+        role5.permissions.add(perm7)
+        role6.permissions.add(perm8)
 
-        # actually role1 doesn't have perm3
-        self.assertFalse(role1._permissions.filter(pk=perm3.pk).exists())
-        # but perm3 is in the subroles of role1 thus add_permissions doesn't add
-        role1.add_permissions(perm3)
-        self.assertFalse(role1._permissions.filter(pk=perm3.pk).exists())
+        self.assertItemsEqual(role1._permissions.all(), [
+                perm1, perm2,
+            ])
+        self.assertItemsEqual(role2._permissions.all(), [
+                perm3,
+            ])
+        self.assertItemsEqual(role3._permissions.all(), [
+                perm4, perm5,
+            ])
+        self.assertItemsEqual(role4._permissions.all(), [
+                perm6,
+            ])
+        self.assertItemsEqual(role5._permissions.all(), [
+                perm7,
+            ])
+        self.assertItemsEqual(role6._permissions.all(), [
+                perm8,
+            ])
 
-        # role2 doesn't have perm2
-        self.assertFalse(role2._permissions.filter(pk=perm2.pk).exists())
-        # and no subroles have perm2 thus add_perms add perm2 to role2
-        role2.add_permissions(perm2)
-        self.assertTrue(role2._permissions.filter(pk=perm2.pk).exists())
 
     def test_remove_permissions(self):
         # role1           -- perm1, perm2
@@ -476,19 +528,57 @@ class PermissionRoleModelTestCase(TestCase):
         role4._permissions.add(perm6)
         role5._permissions.add(perm7)
         role6._permissions.add(perm8)
+        role1.permissions.remove(perm1, perm2)
+        role2.permissions.remove(perm3)
+        role3.permissions.remove(perm4, perm5)
+        role4.permissions.remove(perm6)
+        role5.permissions.remove(perm7)
+        role6.permissions.remove(perm8)
 
-        # actually role1 doesn't have perm3
-        self.assertFalse(role1._permissions.filter(pk=perm3.pk).exists())
-        # thus remove_perm doesn't remove even perm3 is in subroles
-        role1.remove_permissions(perm3)
-        self.assertFalse(role1._permissions.filter(pk=perm3.pk).exists())
-        self.assertTrue(role1.permissions.filter(pk=perm3.pk).exists())
+        self.assertItemsEqual(role1._permissions.all(), [])
+        self.assertItemsEqual(role2._permissions.all(), [])
+        self.assertItemsEqual(role3._permissions.all(), [])
+        self.assertItemsEqual(role4._permissions.all(), [])
+        self.assertItemsEqual(role5._permissions.all(), [])
+        self.assertItemsEqual(role6._permissions.all(), [])
 
-        # role2 have perm3
-        self.assertTrue(role2._permissions.filter(pk=perm3.pk).exists())
-        # thus remove_perm remove perm3
-        role2.remove_permissions(perm3)
-        self.assertFalse(role2._permissions.filter(pk=perm3.pk).exists())
-        self.assertFalse(role2.permissions.filter(pk=perm3.pk).exists())
-        self.assertFalse(role1.permissions.filter(pk=perm3.pk).exists())
+    def test_clear_permissions(self):
+        # role1           -- perm1, perm2
+        #   +- role2      -- perm3
+        #   +- role3      -- perm4, perm5
+        #   |    +- role4 -- perm6
+        #   |    +- role5 -- perm7
+        #   +- role6      -- perm8
+        perm1 = create_permission('permission_test_perm1')
+        perm2 = create_permission('permission_test_perm2')
+        perm3 = create_permission('permission_test_perm3')
+        perm4 = create_permission('permission_test_perm4')
+        perm5 = create_permission('permission_test_perm5')
+        perm6 = create_permission('permission_test_perm6')
+        perm7 = create_permission('permission_test_perm7')
+        perm8 = create_permission('permission_test_perm8')
+        role1 = create_role('permission_test_role1')
+        role2 = create_role('permission_test_role2', role1)
+        role3 = create_role('permission_test_role3', role1)
+        role4 = create_role('permission_test_role4', role3)
+        role5 = create_role('permission_test_role5', role3)
+        role6 = create_role('permission_test_role6', role1)
+        role1._permissions.add(perm1, perm2)
+        role2._permissions.add(perm3)
+        role3._permissions.add(perm4, perm5)
+        role4._permissions.add(perm6)
+        role5._permissions.add(perm7)
+        role6._permissions.add(perm8)
+        role1.permissions.clear()
+        role2.permissions.clear()
+        role3.permissions.clear()
+        role4.permissions.clear()
+        role5.permissions.clear()
+        role6.permissions.clear()
 
+        self.assertItemsEqual(role1._permissions.all(), [])
+        self.assertItemsEqual(role2._permissions.all(), [])
+        self.assertItemsEqual(role3._permissions.all(), [])
+        self.assertItemsEqual(role4._permissions.all(), [])
+        self.assertItemsEqual(role5._permissions.all(), [])
+        self.assertItemsEqual(role6._permissions.all(), [])
